/*
** ===================================================================
** Nzev souboru   : MMIA.c
** Mikrokontrolr  : ATmega8A,...
** Popis           : Hlavn soubor projektu "Generovn zvuku pomoc MCU".
**
** Autor           : Luk Motyka (motylu@seznam.cz)
**				   : Miroslav Prochzka
** Posledn zmna  : 24.04.12 22:59
** ===================================================================
*/


/*
** Hlavikov soubory
*/

#include <avr/io.h>				// hlavikov soubor pro mikrokontrolr
#include <avr/interrupt.h>		// hlavikov soubor pro prci s peruenm
#include <avr/pgmspace.h>		// hlavikov soubor pro prci s pamt FLASH MCU
#ifndef F_CPU
#define F_CPU 8000000UL 		// frekvence krystalu (nutn pro delay.h)
#endif
#include <util\delay.h>     	// hlavikov soubor pro zpodn
#include "stdlib.h"				// hlavikov soubor pro prci s LCD
#include "flash_eeprom.h"		// hlavikov soubor pro prci s extern FLASH EEPROM
#include "lcd_h.h"				// hlavikov soubor pro prci s LCD



/*
** Globln promnn a konstanty v programov pamti
*/

prog_uint16_t sound_page[27] = {0, 36, 82, 126, 181, 223, 270, 314, 363, 408, 461, 517,
						  	    565, 625, 689, 748, 817, 886, 944, 1005, 1056, 1110, 1169,
						  	    1225, 1287, 1356, 1427};
prog_uint16_t sound_byte[27] = {0, 225, 113, 17, 89, 9, 145, 49, 169, 64, 153, 217, 89,
						  	    121, 121, 161, 121, 81, 129, 153, 1, 81, 121, 185, 201,
						        161, 105};
prog_uint16_t sound_length[27] = {9728, 12031, 11519, 14591, 11007, 12543, 11519, 13055,
							      11775, 14080, 14847, 12543, 15871, 16895, 15615, 18175,
							      18175, 15359, 16127, 13311, 14335, 15615, 14847, 16383,
							      18175, 18687, 16895};

volatile uint8_t key = 0;	 	// promnn - hodnota zmknut klvesy
volatile uint8_t key_lock = 0;	// pro "zamknut" klvesnice po stisku tlatka



/*
** Obsluha peruen
*/

ISR (TIMER0_OVF_vect)	// skenovn klvesnice kadch cca 33 ms
{
	if (key_lock == 0)  // pokud je klvesnice "odemen", prove skenovn
	{
		PORTD = 0b00001110;		// 1. sloupec
		if (bit_is_clear(PIND,4)) {key = 3; key_lock = 1;}
		if (bit_is_clear(PIND,5)) {key = 4; key_lock = 1;}
		if (bit_is_clear(PIND,6)) {key = 7; key_lock = 1;}
		if (bit_is_clear(PIND,7)) {key = 10; key_lock = 1;}

		PORTD = 0b00001101;		// 2. sloupec
		if (bit_is_clear(PIND,4)) {key = 1; key_lock = 1;}
		if (bit_is_clear(PIND,5)) {key = 5; key_lock = 1;}
		if (bit_is_clear(PIND,6)) {key = 8; key_lock = 1;}
		if (bit_is_clear(PIND,7)) {key = 0; key_lock = 1;}

		PORTD = 0b00001011;		// 3. sloupec
		if (bit_is_clear(PIND,4)) {key = 2; key_lock = 1;}
		if (bit_is_clear(PIND,5)) {key = 6; key_lock = 1;}
		if (bit_is_clear(PIND,6)) {key = 9; key_lock = 1;}
		if (bit_is_clear(PIND,7)) {key = 10; key_lock = 1;}
	}	
}



/*
** Hlavn program
*/

int main (void)
{
	DDRD = 0x0F;
	PORTD = 0x0F;	// nastaven pin pro maticovou klvesnici
	
	TCCR0 |= 0b00000101;
	TIMSK |= _BV(TOIE0);	// / 0: normln md, peddlika fclk/1024, peruen 
							// pi peteen (skenovn klvesnice kadch cca 33 ms)
	flash_init();
	lcd_init();

	sei();	// globln povolen peruen


	lcd_puts_mov(" Zadejte slo1-99 a potvrte   * nebo #.");
	lcd_puts("Zadno: ");


	uint8_t number = 0, count = 0;	// pomocn promnn

	while (1)	// nekonen smyka programu
	{
		while (!key_lock);	// ekn na "zamen" klvesnice = stisk tlatka

		cli();	// globln zkaz peruen

		if ((key != 10) && (count < 2))		// pokud dosud nebyly zadny 2 slice
		{								 	// a zrove nebylo potvrzeno zadn

			if ((count != 0) || (key != 0))
			{								// pokud nen jako prvn slice
				lcd_putc(key+48);			// stisknuta 0
				count++;					
			}

			switch (count)
			{
				case 1: number += key;
						break;
				case 2: number = number*10 + key;
						break;
				default: break;		// zpis zadvanch sel do pomocn promnn
			}
		}
		else if ((count > 0) && (key == 10))	// pokud byla stisknuta alespo jedna
		{										// platn slice + potvrzen zadn

			if (number < 21)	// pehrn slic 1 a 20
				flash_read(pgm_read_word(&sound_page[number-1]),pgm_read_word(&sound_byte[number-1]),pgm_read_word(&sound_length[number-1]));
			
			else	// pehrn slic 21 a 99
			{
				flash_read(pgm_read_word(&sound_page[(number/10)+17]),pgm_read_word(&sound_byte[(number/10)+17]),pgm_read_word(&sound_length[(number/10)+17]));

				number -= (number/10)*10;

				if (number > 0)
					flash_read(pgm_read_word(&sound_page[number-1]),pgm_read_word(&sound_byte[number-1]),pgm_read_word(&sound_length[number-1]));
			}

			number = 0;
			count = 0;		// vynulovn pomocnch promnnch

			lcd_clrscr();
			lcd_puts("Zadno: ");
		}

		_delay_ms(300);		// zpodn kvli zkmitm a prodlev stisku tlatka

		key_lock = 0;	// optovn "odemen" klvesnice

		sei();	// optovn povolen peruen
	}


	return 1;
}
