/*
** ===================================================================
** Nzev souboru   : lcd_c.c
** Mikrokontrolr  : ATmega16,...
** Popis           : Funkce pro ovldn lcd displeje
**
** Autor           : Peter Fleury ( http://homepage.hispeed.ch/peterfleury/ )
**                   Tom Frza ( fryza@feec.vutbr.cz )
**                   Luk Motyka ( motyckalukas@seznam.cz )
** Pedmt         : Mikroprocesorov technika
** Posledn zmna  : 24.04.12 22:59
** ===================================================================
*/


/*
** Hlavikov soubory
*/
#include <avr\io.h>			// hlavikov soubor pro mikrokontrolr
#ifndef F_CPU
#define F_CPU 8000000UL 	// frekvence krystalu (nutn pro delay.h)
#endif
#include <util\delay.h>     // hlavikov soubor pro zpodn
#include "lcd_h.h"          // hlavikov soubor pro obsluhu lcd


/*
** Globln promnn
*/
unsigned char x,y ;			// pomocn souadnice pozice kurzoru



/*
** ===================================================================
**     Nzev funkce  : lcd_init
**
**     Popis         :
**         Inicializace lcd displeje. 4bitov komunikace; 2 dky;
**         5x8 bod; posun kurzoru doprava; bez posunu displeje;
**         displej zapnut; kurzor vypnut; blikn kurzoru vypnuto
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_init( void )
{
	DDR( LCD_PORT ) |= 0b00111111 ;	// deklarace vstupnho portu pro lcd
    _delay_ms( 16 ) ;				// nutn zpodn
	lcd_command( 0b00100000 ) ; 	// nastaven 4bitov komunikace
	lcd_command( 0b00101000 ) ;		// nastaven funkce displeje
	lcd_command( 0b00000100 ) ;		// nastaven smru pohybu kurzoru

	lcd_clrscr() ;					// nutn smazn displeje


// ****** Vloen novch znak do CGRAM pamti LCD ******

	lcd_command( 1<<LCD_CGRAM  ) ; // nastaven zpisu na prvn pozici v CGRAM

	lcd_data( 0b00000001 ) ;	   // , pozice 0
	lcd_data( 0b00000010 ) ;
	lcd_data( 0b00001110 ) ;
	lcd_data( 0b00000001 ) ;
	lcd_data( 0b00001111 ) ;
	lcd_data( 0b00010001 ) ;
	lcd_data( 0b00001111 ) ;
	lcd_data( 0b00000000 ) ;
												
	lcd_data( 0b00001010 ) ;	   // , pozice 1
	lcd_data( 0b00000100 ) ;
	lcd_data( 0b00001110 ) ;
	lcd_data( 0b00010000 ) ;
	lcd_data( 0b00010000 ) ;
	lcd_data( 0b00010001 ) ;
	lcd_data( 0b00001110 ) ;
	lcd_data( 0b00000000 ) ;

	lcd_data( 0b00001011 ) ;	   // , pozice 2
	lcd_data( 0b00000101 ) ;
	lcd_data( 0b00001101 ) ;
	lcd_data( 0b00010011 ) ;
	lcd_data( 0b00010001 ) ;
	lcd_data( 0b00010001 ) ;
	lcd_data( 0b00001111 ) ;
	lcd_data( 0b00000000 ) ;

	lcd_data( 0b00000001 ) ;	   // , pozice 3
	lcd_data( 0b00000010 ) ;
	lcd_data( 0b00001100 ) ;
	lcd_data( 0b00000100 ) ;
	lcd_data( 0b00000100 ) ;
	lcd_data( 0b00000100 ) ;
	lcd_data( 0b00001110 ) ;
	lcd_data( 0b00000000 ) ;
								                
	lcd_command( LCD_DISP_ON ) ;	// zapnut displeje
	lcd_clrscr() ;					// smazn obsahu displeje (dokonen inicializace)
}


/*
** ===================================================================
**     Nzev funkce  : lcd_clrscr
**
**     Popis         :
**         Smazn obsahu lcd displeje.
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_clrscr( void )
{
	lcd_command( 0b00000001 ) ;	// smazn displeje
	x = 0 ;						// nulovn pomocnch souadnic ...
	y = 0 ;						// ... pozice kurzoru
	_delay_ms( 16 ) ;			// nutn zpodn
}


/*
** ===================================================================
**     Nzev funkce  : lcd_command
**
**     Popis         :
**         Zpis 1bytov instrukce do lcd displeje.
**     Parametry     : data - instrukce pro lcd diplej
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_command( char data )
{
	lcd_write( data, 0 ) ;
}


/*
** ===================================================================
**     Nzev funkce  : lcd_data
**
**     Popis         :
**         Zpis 1bytovho datovho slova do lcd displeje.
**     Parametry     : data - data pro lcd displej
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_data( char data )
{
	lcd_write( data, 1 ) ;
}


/*
** ===================================================================
**     Nzev funkce  : lcd_write
**
**     Popis         :
**         Zpis instrukce nebo dat do lcd displeje.
**     Parametry     : data - instrukce/data pro lcd diplej
**                     rs   - 0: instrukce; 1: data
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_write( char data, char rs )		// pepsno z dvodu pouit menho potu
{											// a jinho rozmstn pin
	unsigned char temp ;

    if( rs ) lcd_rs_high() ;
	else lcd_rs_low() ;

	temp = LCD_PORT & 0x03 ;
    LCD_PORT = temp | (( data & 0xf0 ) >> 2 ) ;
    lcd_e_toggle() ;
    LCD_PORT = temp | (( data & 0x0f ) << 2 ) ;
    lcd_e_toggle() ;
    LCD_PORT = temp | 0xfc;
    _delay_ms( 2 ) ;
}


/*
** ===================================================================
**     Nzev funkce  : toggle_e
**
**     Popis         :
**         Realizace jednoho "enable" pulsu pro lcd displej.
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
void toggle_e( void )
{
    lcd_e_high() ;
    lcd_e_delay() ;
    lcd_e_low() ;
}


/*
** ===================================================================
**     Nzev funkce  : lcd_gotoxy
**
**     Popis         :
**         Pesun kurzoru na pozici x,y lcd displeje.
**     Parametry     : x_,y_ - nov pozice kurzoru lcd displeje
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_gotoxy( char x_, char y_ )
{
	if( y_ == 0 )
    {	lcd_command(( 1<<LCD_DDRAM ) + LCD_START_LINE1 + x_ ) ;
		y = 0 ;
	}
    if( y_ == 1 )
    {	lcd_command(( 1<<LCD_DDRAM ) + LCD_START_LINE2 + x_ ) ;
		y = 1 ;
	}

	x = x_ ;
}


/*
** ===================================================================
**     Nzev funkce  : lcd_putc
**
**     Popis         :
**         Zobrazen jednoho znaku na lcd displeji.
**     Parametry     : c - znak k zobrazen na lcd displeji
**					    - znak pro pesun na dal dek displeje
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_putc( char c )
{
    if ( c=='' )
		lcd_newline() ;
	else
	{
        if(( x == LCD_DISP_LENGTH ) && ( y == 0 ))
			lcd_secondline() ;
		else if(( x == LCD_DISP_LENGTH ) && ( y == 1 ))
			lcd_firstline() ;
	    
		lcd_data( c ) ;
		x++ ;
	}
}


/*
** ===================================================================
**     Nzev funkce  : lcd_puts
**
**     Popis         :
**         Zobrazen etzce znak na lcd displeji.
**     Parametry     : *s - ukazatel na etzec znak
**					     - znak pro pesun na dal dek displeje
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_puts( const char *s )
{
  const char *def_chars = "" ; // nov nadefinovan znaky (azeny dle poad zpisu do CGRAM)
  register char c ;
  int i ;

  while( ( c = *s++ ) )
  { 
    for( i=0;i<4;i++ )		// odesln adresy znaku definovanho v CGRAM
    {
      if( c == def_chars[i] ) c = i ;
    }

	lcd_putc( c ) ;
  }
}


/*
** ===================================================================
**     Nzev funkce  : lcd_puts_mov
**
**     Popis         :
**         Zobrazen pohyblivho (smrem nahoru) etzce znak na lcd displeji.
**     Parametry     : *s - ukazatel na etzec znak
**                       - znak pro pesun na dal dek displeje
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_puts_mov( const char *s )
{
  char temp_mov[17] ;
  int i, m = 0, n = 1, o = 0 ;

  while( s[o] )		// zjitn potu dk v textu
  {
  	if( s[o++] == '' ) n++ ;
  }  

  for( i=1;i<=n;i++ )	// zajitn zpisu vech dk textu
  {
  	o = 0 ;
	lcd_newline() ;
 
    while( ( s[m] != '' ) && ( s[m] ) && ( o < 16 ) ) temp_mov[o++] = s[m++] ; // ten 1 dku

	if( s[m] == '' ) m++ ;	// posunut pozice za znak pro oddkovn	
	temp_mov[o] = 0 ;		// pidn znaku NULL

	lcd_puts( temp_mov ) ;	// vpis dku textu na spodn dek displeje

	_delay_ms( 1e3 ) ;
	lcd_clrscr() ;			// smazn displeje

	lcd_puts( temp_mov ) ;	// vpis stejnho dku textu na vrchn dek displeje
  }

  lcd_secondline() ;

  _delay_ms( 1e3 ) ;
  lcd_clrscr() ;			// konec vpisu textu -> smazn displeje
}


/*
** ===================================================================
**     Nzev funkce  : lcd_newline
**
**     Popis         :
**         Skok na nov dek lcd displeje.
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_newline( void )
{
	if( y == 1 )
		lcd_firstline() ;
	else
		lcd_secondline() ;
}


/*
** ===================================================================
**     Nzev funkce  : lcd_firstline
**
**     Popis         :
**         Skok a vymazn prvnho dku lcd displeje.
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_firstline( void )
{
    lcd_command(( 1<<LCD_DDRAM ) + LCD_START_LINE1 ) ;
	x = 0 ;
	y = 0 ;
	lcd_puts( "                " ) ;
    lcd_command(( 1<<LCD_DDRAM ) + LCD_START_LINE1 ) ;
	x = 0 ;
	y = 0 ;
}


/*
** ===================================================================
**     Nzev funkce  : lcd_secondline
**
**     Popis         :
**         Skok a vymazn druhho dku lcd displeje.
**     Parametry     : dn
**     Vst. hodnota : dn
** ===================================================================
*/
void lcd_secondline( void )
{
    lcd_command(( 1<<LCD_DDRAM ) + LCD_START_LINE2 ) ;
	x = 0 ;
	y = 1 ;
	lcd_puts( "                " ) ;
    lcd_command(( 1<<LCD_DDRAM ) + LCD_START_LINE2 ) ;
	x = 0 ;
	y = 1 ;
}


/*
** Konec zdrojovho souboru
*/
